package controllers

import (
	"errors"
	"net/http"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"

	"github.com/astaxie/beego"
)

// Authenticate 权限验证
func (c *BaseController) authenticate() {
	// 默认 service
	serv := service.NewSysServ(c.Context)

	// 客户端类型
	clientType := utils.GetClientType(c.Ctx.Request)

	// 用户类型, 此处与数据库定义的不是一个概念
	// 此处主要用来区分是 系统用户, 还是 系统客户
	userType := UserGuest
	userTypeRaw := c.GetSession(SNUserType)
	if userTypeRaw != nil {
		userType = userTypeRaw.(string)
	}

	// 用户类型不同, 此处ID代表的内容也是不一样的
	userID := ""
	userIDRaw := c.GetSession(SNUserID)
	if userIDRaw != nil {
		userID = userIDRaw.(string)
	}

	// 用户映射表
	var userMap *model.TaUserMapping

	if clientType == utils.ClientWechat && userID == "" {
		var err error
		userMap, err = c.checkWechat(serv)
		if err != nil {
			utils.LogError("获取并校验微信信息失败: " + err.Error())
		}

		c.SetSession("openid", userMap.Openid)
	}

	if !c.needAuth() {
		return
	}

	// 如果是普通游客, 暂时没有任何设置
	if userType == UserGuest {
		return
	}

	// 如果存在用户映射
	mapUserID := c.getUserIDByUserMap(userMap, userID, userType)
	if mapUserID != "" {
		userID = mapUserID
	}

	// 未登录状态
	if userID == "" {
		if clientType == utils.ClientWechat {
			wxConf, ok := c.Configer[WeChatConf]
			if ok {
				c.ResponseData(
					map[string]interface{}{
						"appid": wxConf.String("appid"),
					},
					errors.New("用户未登录"),
					http.StatusUnauthorized,
				)
			} else {
				c.ResponseError(
					errors.New("用户未登录"),
					http.StatusUnauthorized,
				)
			}
		} else {
			c.ResponseError(
				errors.New("用户未登录"),
				http.StatusUnauthorized,
			)
		}
	}

	// 设置 Session
	c.SetSession(SNUserID, userID)
	c.SetSession(SNUserType, userType)

	var err error

	// 设置 Context
	// 放到 serv 的方法里面了
	if userType == UserAdmin {
		// 普通管理人员
		err = serv.SetUserProfile(userID)

	} else {
		// 系统客户
		err = serv.SetCustomer(userID)
	}

	if err != nil {
		// 暂不处理
	}
}

func (c *BaseController) needAuth() bool {
	route := c.Ctx.Input.URL()
	apiPrefix := beego.AppConfig.String("api::prefix")
	guestAPI := beego.AppConfig.String("api::guest")

	if strings.Index(route, apiPrefix+guestAPI) > -1 {
		return false
	}

	return true
}

func (c *BaseController) checkWechat(serv *service.SysServ) (*model.TaUserMapping, error) {
	// 确定客户端类型
	adminClient := c.GetString("admin")
	userType := service.USERTYPE_CUSTOMER
	if adminClient != "" {
		userType = service.USERTYPE_USER
	}

	openIDRaw := c.GetSession("openid")
	if openIDRaw != nil {
		openID := openIDRaw.(string)
		wxUser := map[string]interface{}{
			"openid": openID,
		}
		return serv.CheckWechatUserMapping(wxUser, userType)
	}

	// 微信 code
	code := c.GetString("code")
	if code == "" {
		return nil, errors.New("没有微信 code")
	}

	// 获取微信信息
	wxUser, err := utils.GetWxUserInfo(code)
	if err != nil {
		return nil, err
	}

	return serv.CheckWechatUserMapping(wxUser, userType)
}

// getUserIDByUserMap 校验用户映射是否与 Session 一致
func (c *BaseController) getUserIDByUserMap(user *model.TaUserMapping, userID, userType string) string {
	// 如果映射信息不存在
	if user == nil {
		return ""
	}

	// 如果 session 数据存在, 以 session 为准
	if userID != "" {
		return ""
	}

	// 如果是系统内置用户
	if user.UserType == service.USERTYPE_USER {
		if userType != UserAdmin {
			return ""
		}
	}

	// 如果是普通客户
	if user.UserType == service.USERTYPE_CUSTOMER {
		if userType != UserCustomer {
			return ""
		}
	}

	return user.UserId
}
